@@ -64,6 +64,7 @@ class ServiceTemplateGenerator(TemplateGenerator):
         self.bucket_name = 'cloudlift-service-template'
         self.environment = service_configuration.environment
         self.client = get_client_for('s3', self.environment)
+        self.team_name = (self.notifications_arn.split(':')[-1])
 
     def _derive_configuration(self, service_configuration):
         self.application_name = service_configuration.service_name
@@ -193,6 +194,7 @@ service is down',
             "Image": self.ecr_image_uri + ':' + self.current_version,
             "Essential": 'true',
             "LogConfiguration": self._gen_log_config(service_name),
+            "Memory": int(config['memory_reservation']) + -(-(int(config['memory_reservation']) * 50 )//100), # Celling the value 
             "MemoryReservation": int(config['memory_reservation']),
             "Cpu": 0
         }
@@ -239,7 +241,7 @@ service is down',
                 'Cpu': str(config['fargate']['cpu']),
                 'Memory': str(config['fargate']['memory'])
             }
-        
+
         if 'custom_metrics' in config:
             launch_type_td['NetworkMode'] = 'awsvpc'
         if 'volume' in config:
@@ -256,7 +258,9 @@ service is down',
             Family=service_name + "Family",
             ContainerDefinitions=[cd],
             TaskRoleArn=Ref(task_role),
+            Tags=Tags(Team=self.team_name, environment=self.env),
             **launch_type_td
+            
         )
         if 'custom_metrics' in config:
             sd = SD(
@@ -289,6 +293,7 @@ service is down',
             if launch_type == self.LAUNCH_TYPE_FARGATE:
                 # if launch type is ec2, then services inherit the ec2 instance security group
                 # otherwise, we need to specify a security group for the service
+                launch_type_svc = {}
                 if 'custom_metrics' in config:
                     launch_type_svc = {
                         "ServiceRegistries": [ServiceRegistry(
@@ -308,7 +313,8 @@ service is down',
                             'FromPort': int(config['http_interface']['container_port']),
                         }],
                         VpcId=Ref(self.vpc),
-                        GroupDescription=pascalcase("FargateService" + self.env + service_name)
+                        GroupDescription=pascalcase("FargateService" + self.env + service_name),
+                        Tags=Tags(Team=self.team_name, environment=self.env)
                     )
                     self.template.add_resource(service_security_group)
 
@@ -350,7 +356,7 @@ service is down',
                         'Role': Ref(self.ecs_service_role),
                         'PlacementStrategies': self.PLACEMENT_STRATEGIES
                     }
-            
+
             svc = Service(
                 service_name,
                 LoadBalancers=[lb],
@@ -360,6 +366,7 @@ service is down',
                 DependsOn=service_listener.title,
                 LaunchType=launch_type,
                 **launch_type_svc,
+                Tags=Tags(Team=self.team_name, environment=self.env)
             )
             self.template.add_output(
                 Output(
@@ -407,7 +414,8 @@ service is down',
                         GroupName=pascalcase("FargateService" + self.env + service_name),
                         SecurityGroupIngress=[],
                         VpcId=Ref(self.vpc),
-                        GroupDescription=pascalcase("FargateService" + self.env + service_name)
+                        GroupDescription=pascalcase("FargateService" + self.env + service_name),
+                        Tags=Tags(Team=self.team_name, environment=self.env)
                     )
                     self.template.add_resource(service_security_group)
                     launch_type_svc = {
@@ -456,7 +464,8 @@ service is down',
                 DesiredCount=desired_count,
                 DeploymentConfiguration=deployment_configuration,
                 LaunchType=launch_type,
-                **launch_type_svc
+                **launch_type_svc,
+                Tags=Tags(Team=self.team_name, environment=self.env)
             )
             self.template.add_output(
                 Output(
@@ -487,7 +496,8 @@ service is down',
                 config
             ),
             VpcId=Ref(self.vpc),
-            GroupDescription=Sub(service_name + "-alb-sg")
+            GroupDescription=Sub(service_name + "-alb-sg"),
+            Tags=Tags(Team=self.team_name, environment=self.env)
         )
         self.template.add_resource(svc_alb_sg)
         alb_name = service_name + pascalcase(self.env)
@@ -508,7 +518,9 @@ service is down',
                 ],
                 Name=alb_name,
                 Tags=[
-                    {'Value': alb_name, 'Key': 'Name'}
+                    {'Value': alb_name, 'Key': 'Name'},
+                    {"Key": "Team", "Value": self.team_name},
+                    {'Key': 'environment', 'Value': self.env}
                 ],
                 Scheme=scheme
             )
@@ -527,7 +539,9 @@ service is down',
                 ],
                 Name=alb_name,
                 Tags=[
-                    {'Value': alb_name, 'Key': 'Name'}
+                    {'Value': alb_name, 'Key': 'Name'},
+                    {"Key": "Team", "Value": self.team_name},
+                    {'Key': 'environment', 'Value': self.env}
                 ]
             )
 
@@ -559,7 +573,11 @@ service is down',
             Port=int(config['http_interface']['container_port']),
             HealthCheckTimeoutSeconds=10,
             UnhealthyThresholdCount=3,
-            **target_group_config
+            **target_group_config,
+            Tags=[
+                {"Key": "Team", "Value": self.team_name},
+                {'Key': 'environment', 'Value': self.env}
+            ]
         )
 
         self.template.add_resource(service_target_group)
